import re
from types import FunctionType,MethodType # 对象的叫方法[method]，其他未函数[Function]

import inspect

def get_current_function_name():
    return inspect.currentframe().f_back.f_code.co_name

def to_camelcase(name: str) -> str:
    """
    下划线转驼峰(小驼峰)
    """
    return re.sub(r'(_[a-z])', lambda x: x.group(1)[1].upper(), name)
def to_underscore(name: str) -> str:
    """
    驼峰转下划线
    """
    if '_' not in name:
        name = re.sub(r'([a-z])([A-Z])', r'\1_\2', name)
    else:
        raise ValueError(f'{name}字符中包含下划线，无法转换')
    return name.lower()

def list_to_tree( data_list:list, root:any, pid_field:str, id_field:str):
    """
    list 转 tree 

    data_list: 数据列表,
    root: 通过 `.get(pid_field) == root ` 查出所有根节点, 或者是找到 根的 一个方法
    pid_field: 关联父节点的字段,
    id_field:  主键id

    return 树形 包含 children 字段
    """ 
    getRoot=root
    def _getRoot(a__data_list:dict):
        return a__data_list.get(pid_field)== root

    if not isinstance(root,FunctionType):
        getRoot=_getRoot

    resp_list = [i for i in data_list if getRoot(i)]  
    for i in data_list:
        i['children'] = [j for j in data_list if i.get(id_field) == j.get(pid_field)] 
    return resp_list 


"""
https://docs.python.org/zh-cn/3/library/unittest.html
python -m unittest test_module1 test_module2
python -m unittest test_module.TestClass
python -m unittest test_module.TestClass.test_method
python -m unittest tests/test_something.py
"""

import unittest
 
def test_example ():
    pass
 
if __name__ == '__main__': 
    unittest.main()


# -*- coding:utf-8 -*-
import re
import random
import string
import time
import datetime
from types import FunctionType
import inspect
import os
import io
import sys
from typing import IO, Iterable, Callable, Tuple, Generator
from .log import warn


def isBase64(content: str) -> bool:
    _reg = "^([A-Za-z0-9+/]{4})*([A-Za-z0-9+/]{4}|[A-Za-z0-9+/]{3}=|[A-Za-z0-9+/]{2}==)$"
    group = re.match(_reg, content)
    if group != None:
        return True
    return False


def debug():
    frameList = inspect.stack()
    sss = ["{}:{} ->{}".format(i.filename, i.lineno, i.function) for i in frameList]
    print(len(frameList))
    warn('\n' + '\n'.join(sss))


def getRandomStr(length: int, scope: str = string.ascii_letters+string.digits) -> str:
    return ''.join(random.sample(scope, length))


def generate_id(*_):
    return time.time_ns()


def getDateFolder(format: str = "%Y/%m/%d"):
    """
    获得当前日期目录:
    2023/12/01
    """
    time = datetime.datetime.now()
    return f"{time.strftime(format)}"


def isCallable(func):
    return isinstance(func, FunctionType)
    return callable(func)  # 返回true 也不一定能调用成功/返回失败一定调用失败
    return type(func) is FunctionType
    return hasattr(func, "__call__")


def is_async(func):
    """
    方法是否是异步的
    """
    return inspect.iscoroutinefunction(func) or inspect.isasyncgenfunction(func)


def getWorkDirectory():
    """
    获取工作目录
    """
    current_directory = os.getcwd()
    return current_directory


def find_files(root, *ignoreDirs: str, filterDirFunction: Callable[[str], bool] = None, filterFileFunction: Callable[[str], bool] = None) -> Generator[Tuple[str, list, list], str]:
    """
    root: 根目录
    filterDirFunction  (folderName:str)->bool 
    filterFileFunction (fileName:str)->bool
                       可以使用  lambda f:f.endswith(extension)
    RETURN  root  fdirs, ffile 
    查找文件或文件夹
    os.path.join(root, file)

    """
    for root, dirs, files in os.walk(root):
        for ignore in ignoreDirs:
            if ignore in dirs:
                dirs.remove(ignore)
        fdirs = list(filter(filterDirFunction, dirs)) if filterDirFunction != None else dirs
        ffile = list(filter(filterFileFunction, files)) if filterFileFunction != None else files
        yield root, fdirs, ffile


def get_parent_dir(path: str, rank: int = 1):
    """
    获取上级目录
    """
    result: str = path
    stop = rank+1
    for _ in range(1, stop):
        result = os.path.dirname(result)
    return result


def getApplcationPath(__file__):
    # print("如果脚本被编译成.pyc文件运行或者使用了一些打包工具（如PyInstaller），那么__file__可能不会返回源.py文件的路径，而是编译后的文件或临时文件的路径")
    # 获取当前文件的完整路径
    if getattr(sys, 'frozen', False):
        # 如果应用程序是冻结的，获取可执行文件的路径
        application_path = os.path.dirname(sys.executable)
    else:
        # 否则，获取原始脚本的路径
        application_path = os.path.dirname(os.path.abspath(__file__))

    return application_path


def read_stream(stream: IO[bytes], size: int = -1) -> Iterable[bytes]:
    while True:
        chunk = stream.read(size)
        if not chunk:
            break
        yield chunk


async def write_stream(input: IO[bytes], outputStream: IO[bytes]):
    for chunk in read_stream(input, size=io.DEFAULT_BUFFER_SIZE):
        try:
            # print("**FF读数据", len(chunk))
            outputStream.write(chunk)
            outputStream.flush()
            # print("**输出到stdin*****",len(chunk))
        except Exception as e:
            pass
            # print("**FF异常", e, len(chunk))
    outputStream.close()



# -*- coding:utf-8 -*-


import json,datetime
from typing import Any

class JSON_util(json.JSONEncoder):
    def default(self, obj: Any) -> Any:  
        if isinstance(obj, datetime.datetime):  return obj.strftime("%Y-%m-%d %H:%M:%S")
        elif isinstance(obj, bytes): return str(obj, encoding='utf-8')
        elif isinstance(obj, int): return int(obj)
        elif isinstance(obj, float):return float(obj) 
        elif hasattr(obj,"__dict__"):  return obj.__dict__ 
        return None 
    


# -*- coding:utf-8 -*-

import datetime
import sys
import threading
import os
# from loguru import logger
import logging
import logging.handlers
import traceback
from pathlib import Path
from typing import List
'''
loguru 日志配置
'''
# logger.remove()
# logger.add(sys.stdout,level="INFO",format="{file}\t{line}\t{message}")
'''
#https://blog.csdn.net/Kangyucheng/article/details/112794185
TRACE           5   trace()
DEBUG           10  debug()
INFO            20  info()
SUCCESS         25  success()
WARNING         30  warning()
ERROR           40  error()
CRITICAL        50  critical()
'''
 
def create_logger(filename: str,folder:str=".",showLevel:int=logging.DEBUG, format:str="%(asctime)s %(levelname)5.5s: %(message)s") -> logging.Logger:
    logger = logging.getLogger(__name__)
    logger.setLevel(showLevel)
    #folder=Path(__file__).parent.joinpath("logs")
    if not os.path.exists(folder):
        os.makedirs(folder)
    formatter = logging.Formatter(format)
    handlers:List[logging.Handler] = [
        logging.handlers.RotatingFileHandler( Path(folder).joinpath(  filename), maxBytes=1024*1024, backupCount=0,encoding='utf-8'),
        logging.StreamHandler()
    ]
    for handler in handlers:
        handler.setFormatter(formatter)
        logger.addHandler(handler)
    return logger

'''
控制台日志 日志配置
'''
"""
\033[显示方式;前景色;背景色m*******\033[0m
\033[显示方式;前景色;背景色m*******\033[0m
显示方式:    0:默认值
            1:高亮
            4:下划线
            5:闪烁
            7:反显
            8:不可见
控制台颜色值：
前景色      背景色      颜色说明
 30           40        黑色
 31           41        红色
 32           42        绿色
 33           43        黄色
 34           44        蓝色
 35           45        紫红色
 36           46        青蓝色
 37           47        白色
"""


def __getMessage(*msg: str):
    return "\t".join([str(m) for m in msg])


def __log(*msg: str, type: int = 0, foregroundColor: int = 37, bg=40, e=None, hasPrefix: bool = True):
    t = threading.currentThread()
    time = datetime.datetime.now()
    err = e.__traceback__.tb_lineno if e != None else ""
    prefix = f"['{time.strftime('%Y-%m-%d %H:%M:%S')}'] [{t.ident}|{t.name}]\t"
    if not hasPrefix:
        prefix = ""
    # __getMessage(*msg).encode("utf-8") 以前为什么不显示乱码
    print(f"{prefix}\033[{type};{foregroundColor};{bg}m{__getMessage(*msg)}{err}\033[0m")


def log(*msg: str): __log(*msg)


def generateCode(data: any):
    """
    通过 对象 转字符串 
    生成  大小  hash值
    """
    s = str(data)
    return hex(s.__hash__()).upper()


def start_mark(msg: str, f: str = "--", start: str = "\r\n", end: str = ">", num: int = 36):
    """
    标记开始
    """
    __log(start+f*num + msg + f*num+end, hasPrefix=False)


def end_mark(msg: str, f: str = "==", start: str = "\r\n<", end: str = "\r\n\r\n", num: int = 36):
    """
    标记结束
    """
    __log(start+f*num + msg + f*num+end, hasPrefix=False)


def info(*msg: str): __log(*msg)


def succ(*msg: str):
    """
    成功日志
    """
    __log(*msg, type=7, foregroundColor=32, bg=40)


def warn(*msg: str):
    """
    警告日志
    """
    __log(*msg, type=7, foregroundColor=33, bg=40)


def err(*msg: str, e=None):
    """
    错误日志
    """
    message = __getMessage(*msg)
    __log(f'{message}-->{traceback.format_exc()}', type=7, foregroundColor=31, bg=40, e=e)


def critical(msg: str):
    """
    危急日志
    """
    message = __getMessage(*msg)
    __log(f'{message}-->{traceback.format_exc()}', type=7, foregroundColor=37, bg=40)


'''
if __name__ == "__main__" :
    logger.trace("文件日志")
    logger.success("*arg:{}{}\t\t**kwargs:'ab:{ab},cd:{cd}'","元数据1","元数据2",ab="字典数据1",cd="字典数据2")
'''


def progress_bar(i: float, title: str = ""):
    """
    进度条
    """
    i = int(i*100)
    print("\r", end="")
    print("{}: {}%: ".format(title, i), "▋" * (i // 2), end="")
    sys.stdout.flush()



# -*- coding:utf-8 -*-

import json,base64,os
from  co6co.utils import hash

class File:
    
    @staticmethod
    def readFile2Base64(filePath:str):
        """
        文件内容  -->  base64
        """
        with open(filePath, 'rb') as file:
            file_content = file.read()
            return hash.enbase64(file_content)
    @staticmethod
    def readFileBase642Str(filePath:str):
        """
        文件内容base64 --> content  
        """
        with open(filePath, 'rb') as file:
            file_content = file.read()
            return hash.debase64(file_content)
        
    @staticmethod
    def readJsonFile(filePath:str, encoding:str="utf-8"):
        """
        filePath: 文件路径
        encoding:文件编码
        return     json 对象
        """
        with open(filePath, "r", encoding=encoding) as f:
            result = json.load(f) 
            return result
    
    @staticmethod
    def writeJsonFile(filePath,obj):
        """
        filePath: 文件路径
        obj : 待写入的对象
        """
        updated_list = json.dumps( obj, sort_keys=False, indent=2, ensure_ascii=False)
        with open(filePath, 'w', encoding='utf-8') as file:
            file.write(updated_list) 

    @staticmethod
    def readFile(filePath: str, encoding: str = "utf-8") -> str:
        """
        filePath: 文件路径
        encoding:文件编码
        return content 
        """
        with open(filePath,"r",encoding=encoding) as file: 
            content=file.read() #.splitlines()# readlines() 会存在\n 
            return content
    @staticmethod
    def createFolder(filePath:str,isFilePath:False)->str:
        """
        创建路径,并返回 传入的路径 以当前操作系统 分隔路径符号分割
        """
        filePath=os.path.abspath(filePath) # 转换为 os 所在系统路径 
        folder=filePath
        if isFilePath: folder=os.path.dirname(filePath) 
        if not os.path.exists(folder):os.makedirs(folder)
        return filePath
    
    @staticmethod
    def writeFile(filePath,data:bytes):
        """
        filePath: 文件路径
        obj : 待写入的对象
        """ 
        filePath=File.createFolder(filePath,True)
        with open(filePath,"wb") as file: #二进制模式打开文件
            file.write(data)
    @staticmethod
    def writeBase64ToFile(filePath,base64Content:str):
        """
        filePath: 文件路径
        obj : 待写入的对象
        """
        data=base64.b64decode(base64Content)
        File.writeFile(filePath,data)
            


# -*- encoding:utf-8 -*-
# 动态代码
from typing import Callable


def read_file(file_path):
    with open(file_path, "rb") as f:
        source_code = f.read()
        return source_code


"""
source: 字符串形式的 Python 代码或 AST 对象。这是要编译的源代码。
filename: 字符串。如果源代码来自文件，则提供该文件的名称；如果不是来自文件，可以提供一个合适的名称，如 '<string>' 表示源代码是直接以字符串形式提供的。
mode: 编译模式，可以是 'exec', 'eval', 或 'single'。这决定了源代码应该如何被解释：
    'exec': 源代码包含一个或多个语句。
    'eval': 源代码是一个表达式。
    'single': 类似于 'exec'，但如果源代码只有一行，则会像在交互式解释器中那样执行。
flags (可选): 编译标志，用于指定特定的编译选项。这是一个整数，默认值为 0。例如，可以使用 PyCF_ONLY_AST 标志来返回 AST 而不是编译后的代码对象。
dont_inherit (可选): 如果设置为 True，那么编译过程中不会继承全局变量。默认值为 False。
optimize (可选): 优化级别。默认值为 -1，表示使用命令行指定的优化级别。其他可能的值包括 0（无优化）、1 和 2。
"""


def get_source_fun(source_code, func_name) -> Callable[[], None]:
    """
    source_code: 代码
    func_name: 代码中的方法
    """
    scope = {}
    compile_source(source_code, scope)
    f = scope.get(func_name, None)
    return f


def compile_source(source_code, global_vars: dict = None, local_vars: dict = None):
    """
    source_code: 代码 
    """
    exec_code = compile(source_code, '<string>', "exec")
    exec(exec_code, global_vars, local_vars)



from enum import Enum, unique
# from abc import ABC, abstractclassmethod  抽象
from typing import List, Dict

from co6co import T


@unique  # 帮助检查 保证没有重复值
class Base_Enum  (Enum):
    """
    枚举[key, val]
    """
    key: T
    val: T

    def __new__(cls, key: T, value: T):
        obj = object.__new__(cls)
        obj.key = key
        obj.val = value  # value 为元组 (en_name,cn_name,val)
        return obj

    @classmethod
    def to_dict_list(cls) -> List[Dict]:
        status = [{'uid': i.name, 'key': i.key, 'value': i.val} for i in cls]
        return status 
    
    @classmethod
    def key2enum(cls, key):
        """
        key 转枚举 
        """
        for i in cls:
            if i.key==key:return i
        return None
    
    @classmethod
    def val2enum(cls, value):
        """
        val 转枚举 
        """
        for i in cls:
            if i.val==value:return i
        return None
    
    @classmethod
    def value_of(cls, value):
        """
        枚举的字符串 转枚举
        demo(Base_Enum):
            chanel="ch",1
        字符串 为 chanel 
        """
        for k, v in cls.__members__.items():
            if k == value:
                return v
        else:
            raise ValueError(f"'{cls.__name__}' enum not found for '{value}'")

    def getValue(self) -> T:
        return self.val

    def getKey(self) -> T:
        return self.key


@unique
class Base_EC_Enum(Enum):
    """
    枚举[key:英文 ,name:中文 ,val:数字] 
    """
    key: T
    label: T
    val: T

    def __new__(cls, key: T, label: T, value: T):
        obj = object.__new__(cls)
        obj.key = key
        obj.label = label
        obj.val = value  # value 为元组 (en_name,cn_name,val)
        return obj

    @classmethod
    def to_dict_list(cls) -> List[Dict]:
        status = [{'uid': i.name, "key": i.key,
                   'label': i.label, 'value': i.val} for i in cls]
        return status

    def getValue(self) -> T:
        return self.val

    def getKey(self) -> T:
        return self.key

    def getLabel(self) -> T:
        return self.label



# -*- coding:utf-8 -*-
from typing import TypeVar, List
T = TypeVar('T')


class Base:
    def __repr__(self) -> str:
        return f'{self.__class__}'


def byteSite(bytesLength: int):
    """ 
    (B,KB,MB,GB,TB)
    """
    kb = round(bytesLength/1024, 2)
    mb = round(bytesLength/1024**2, 2)
    gb = round(bytesLength/1024**3, 2)
    tb = round(bytesLength/1024**4, 2)
    return bytesLength, kb, mb, gb, tb


def getByteUnit(bytesLength: int):
    kmb = byteSite(bytesLength)
    unit = ["B", "KB", "MB", "GB", "TB"]
    f = 0
    # 逆序
    # reversed_arr = kmb[::-1]
    for s in kmb:
        if (s < 1024):
            return f"{s}{unit[f]}"
        f += 1
    return None


__all__ = ['utils']  # 针对模块公开接口的一种约定，以提供了”白名单“的形式暴露接口。
# 如果定义了__all__，
# 使用from xxx import *导入该文件时，只会导入 __all__ 列出的成员，可以其他成员都被排除在外。

__version_info = (0, 0, 15)
__version__ = ".".join([str(x) for x in __version_info])




from __future__ import annotations
from datetime import datetime
from typing import Type, TypeVar

T = TypeVar('T')


def singleton(cls):
    """
    单例模式装饰器

    使用方法：
    @singleton
    class MyClass:
        pass
    """
    instances = {}

    def get_instance(*args, **kwargs):
        if cls not in instances:
            instances[cls] = cls(*args, **kwargs)
            instances[cls].createTime = datetime.now()
        return instances[cls]
    return get_instance


class Singleton:
    """
    单例模式 子类继承
    """
    _instance: T = None
    createTime = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(Singleton, cls).__new__(cls, *args, **kwargs)
        return cls._instance

    def __init__(self) -> None:
        self.createTime = datetime.now()
        pass




from fractions import Fraction


def dms_to_decimal(degrees: float, minutes: float, seconds: float, direction: str = None):
    """
    将度、分、秒格式的坐标转换为十进制分数格式。

    :param degrees: 度数 (float)
    :param minutes: 分数 (float)
    :param seconds: 秒数 (float)
    :param direction: 方向 ('N', 'S', 'E', 'W')，可选参数
    :return:  分数形式可以防止精度丢失，十进制分数格式的坐标 float(1/100) 将分数转小数
    """
    degrees_fraction = Fraction(degrees)
    minutes_fraction = Fraction(minutes, 60)
    seconds_fraction = Fraction(seconds, 3600)

    decimal_value = degrees_fraction + minutes_fraction + seconds_fraction
    # 根据方向调整正负号
    if direction in ['S', 'W']:
        decimal_value = -decimal_value

    return decimal_value



# -*- coding: utf-8 -*-
import unittest

from co6co.utils import hash
from co6co.utils import File
from co6co.utils import log

class TestFile(unittest.TestCase):
    content:str=None
    def setUp(self):
        self.content="co6co[安安]！"
        pass 
    def test_base64(self): 
        data=hash.enbase64(self.content)
        self.assertEqual(data , 'Y282Y29b5a6J5a6JXe+8gQ==') 
        print(type(data))
        self.assertEqual(hash.debase64(data),self.content)
        
    def test_md5(self):
        self.assertEqual(hash.md5(self.content),"8e95c4650cf629db1a94dd1e0a72e787") 
    def test_hash(self):
        self.assertEqual(hash.str_sha1(self.content),"ae4fe3db3c0280941d34be63e8c0af330f9e5e5a")
        self.assertEqual(hash.str_sha224(self.content),"920eb220e67336c3ecb5d4b2b25fe10ec0651a15e050ee4355cc2f02")
        self.assertEqual(hash.str_sha256(self.content),"ad13bb193d5cd19c29fd8e6a70ae7679b931497c0f35116aed8c101659a9890f")
        self.assertEqual(hash.str_sha384(self.content),"45c7b80604d68f5e1e5c2b994cdec2ac2b8d42d7d3dcd0676151ecd85913cb37ea4497fff14a13ad38a95bf7cfa93787")
        self.assertEqual(hash.str_sha512(self.content),"e319fdd751fa61e08b5874e94ce80e2e9ff85fad0fe5ad2e96cd2ebf43a14a572803f95b359b21933ac4acf0f30baa1f3d15336748b28763e77c4951e6692d5c")
        
     
        
    def tearDown(self):
        pass
    
def suite():
    #若测试用例都是以test开头，则用 makeSuite()方法一次性运行
    return unittest.makeSuite(TestFile )
    suite=unittest.TestSuite() 
    suite.addTest(TestFile("test_base64"))  #添加测试用例
    return suite

if __name__=="__main__":
    #文件使用方案1. python v_utils_test.py
    #文件使用方案2 python -m unittest v_utils_test
    # 使用方案1
    unittest.main(defaultTest='suite') 
    # 使用方案2
    #runner=unittest.TextTestRunner()
    #runner.run(suite2)
    



from concurrent.futures import ThreadPoolExecutor
import queue


class limitThreadPoolExecutor(ThreadPoolExecutor):
    """
    限制进程池队列长度（默认队列长度无限）
    防止内存爆满的问题

    """

    def __init__(self, max_workers=None, thread_name_prefix=''):
        super().__init__(max_workers, thread_name_prefix)
        # 不甚至将时无限队列长度
        self._work_queue = queue.Queue(self._max_workers * 2)  # 设置队列大小



import asyncio
from threading import Thread
from time import sleep, ctime
from co6co.utils import log
from functools import partial
from types import FunctionType
from typing import Callable
from co6co.utils import isCallable


class ThreadEvent:
    """
    线程Event loop
    Run Event Loop in different thread.

    ## 因某些原因写了该类
    ## 1. asyncio.run 在 没有正在运行的事件循环 的情况下运行协程的
        ## wx_user_dict:list[dict]=asyncio.run(bll.get_subscribe_alarm_user(config.appid))
        ## alarm= alarm_bll(app)
        ## po:bizAlarmTypePO=asyncio.run(alarm.get_alram_type_desc(po.alarmType))
        ## 2. 正在运行的事件循环
        ## This event loop is already running
        ## import nest_asyncio
        ## loop = asyncio.get_event_loop()
        ## wx_user_dict:list[dict]=loop.run_until_complete(bll.get_subscribe_alarm_user(config.appid))

        ## 3. 创建任务
        ## task=asyncio.create_task(bll.get_subscribe_alarm_user(config.appid))
        ## wx_user_dict:list[dict]=asyncio.run(task)
        ## 4.底层使用
        ## asyncio.ensure_future(coro())
        ## 5.
    """
    @property
    def loop(self):
        return self._loop

    bck: FunctionType = None

    def __init__(self, threadName: str = None, quitBck: FunctionType = None):
        self._loop = asyncio.new_event_loop()
        # log.warn(f"ThreadEventLoop:{id(self._loop)}")
        Thread(target=self._start_background, daemon=True, name=threadName) .start()
        self.bck = quitBck

    def _start_background(self):
        asyncio.set_event_loop(self.loop)
        self._loop.run_forever()
        if self.bck != None and isCallable(self.bck):
            self.bck()

    def runTask(self, tastFun, *args, **kwargs):
        # log.warn(f"ThreadEventLoop22:{id(self._loop)}")
        task = asyncio.run_coroutine_threadsafe(tastFun(*args, **kwargs), loop=self._loop)
        return task.result()

    def _shutdown(self):
        self._loop.stop()
        # print("running:", self._loop.is_running()) #True
        # print("closed.", self._loop.is_closed())   #False

    def close(self):
        self._loop.call_soon_threadsafe(partial(self._shutdown))

    def __del__(self):
        self._loop.close()
        # log.warn("closed")
        # print("running:", self._loop.is_running()) #False
        # print("closed.", self._loop.is_closed())   #True


class EventLoop:
    """
    数据库操作
    定义异步方法
    运行 result=run(异步方法,arg)

    """
    _eventLoop: ThreadEvent
    _closed: bool = None

    def __init__(self) -> None:
        self._eventLoop = ThreadEvent()
        self._closed = False

    def run(self, task, *args, **argkv):
        if self._closed:
            raise RuntimeError('ThreadEvent is closed')
        data = self._eventLoop.runTask(task, *args, **argkv)
        return data

    def close(self):
        self._eventLoop.close()
        self._closed = True

    def __del__(self) -> None:
        if not self._closed:
            self._eventLoop.close()


class Executing:
    """
    线程 自己执行自己退出
    """
    _starting: bool = None

    @property
    def loop(self):
        return self._loop

    @property
    def runing(self):
        return self._starting

    bck: FunctionType = None
    args = None
    kvgs = None

    def __init__(self, threadName: str, func,   *args, **kvgs):
        '''
        threadName: 线程名
        func: 执行的方法 async   :Callable[[str], str]
        args:  func 参数
        kvgs: func 参数
        '''
        self._loop = asyncio.new_event_loop()
        self._isCallClose = False
        self.threadName = threadName
        self.bck = func
        self.args = args
        self.kvgs = kvgs
        Thread(target=self._start_background, daemon=True, name=threadName) .start()

        def _start_background(self):
            try:
                asyncio.set_event_loop(self.loop)
                log.log("线程'{}->{}'运行...".format(self.threadName, id(self.loop)))
                self.loop.run_until_complete(self.bck(*self.args, **self.kvgs))
                # await self.bck(*self.args,**self.kvgs)
            except Exception as e:
                log.warn("线程'{}->{}'执行出错:{}".format(self.threadName, id(self.loop), e))
            finally:
                log.log("线程'{}->{}'结束.".format(self.threadName, id(self.loop)))
                self.loop.close()

    class TaskManage:
        _starting: bool = None

    @ property
    def loop(self):
        return self._loop

    @ property
    def runing(self):
        return self._starting

    bck: FunctionType = None

    def __init__(self, threadName: str = None):
        self._loop = asyncio.new_event_loop()
        self._isCallClose = False
        self.threadName = threadName
        Thread(target=self._start_background, daemon=True, name=threadName) .start()

    def _start_background(self):
        asyncio.set_event_loop(self.loop)
        log.log("线程'{}->{}'运行...".format(self.threadName, id(self.loop)))
        self._starting = True
        self._loop.run_forever()
        log.log("线程'{}->{}'结束.".format(self.threadName, id(self.loop)))
        self._starting = False

    def runTask(self, tastFun, callBck, *args, **kwargs):
        """
        不能回调 调用 close 因为还在执行中.
        """
        # log.warn(f"ThreadEventLoop22:{id(self._loop)}")
        # run_coroutine_threadsafe 从非事件循环线程向事件循环线程提交协程任务
        task = asyncio.run_coroutine_threadsafe(tastFun(*args, **kwargs), loop=self._loop)
        # .result() 方法等待协程的结果，或者使用 .add_done_callback() 添加回调来处理结果。
        if callBck != None:
            task.add_done_callback(callBck)
        else:
            return task.result()

    def _stop(self):
        self._loop.stop()
        self._starting = False

    def stop(self):
        """
        runTask: 执行完后后才能调用，可再回调中调用
        调用完记得关闭
        """
        self._loop.call_soon_threadsafe(self._stop)

    def close(self):
        self._loop.call_soon_threadsafe(self._loop.close)
        self._isCallClose = True

    def __del__(self):
        if not self._isCallClose:
            self._loop.close()
